﻿Imports System.Text.RegularExpressions


Public Enum TokenType
    [Operator]
    [Function]
    Number
End Enum
Public Enum Associvity
    Left
    Right
    None
End Enum
Public Enum Mode
    Radian
    Degree
End Enum
Public Enum PointType
    High
    Low
    Saddle
End Enum
Public Enum Symmetry
    Origin
    Axis
    None
End Enum

Public Enum CurveChange
    RightToLeft
    LeftToRight
    None
End Enum

Public Class FunctionPoint
    Public Property X As Double
    Public Property Y As Double

    Public Sub New(ByVal x As Double, ByVal y As Double)
        _X = x
        _Y = y
    End Sub
    Public Overrides Function ToString() As String
        Return String.Concat({"(", X.ToString, "|", Y.ToString, ")"})
    End Function
End Class

Public Class ExtremePoint
    Dim _type As PointType
    Public Property Type As PointType
        Get
            Return _type
        End Get
        Set(value As PointType)
            _type = value
        End Set
    End Property
    Public Property Point As FunctionPoint

    Public Overrides Function ToString() As String
        Return String.Join(" ", {Type.ToString, Point.ToString})
    End Function
End Class


Public Class InflectionPoint


    Public Property Point As FunctionPoint

    Public Property CurveChange As CurveChange


    Public Overrides Function ToString() As String
        Return Point.ToString & " " & CurveChange.ToString
    End Function
End Class



Public NotInheritable Class Parser
    Shared Functions As String()
    Shared Operators As String()
    Shared ValidTokens As String()
    Public Property TrigonometricMode As Mode

    ''' <summary>
    ''' Instantiation of a new Parser Class
    ''' </summary>
    ''' <param name="trigonometric_mode">The way the parser evaluates trigonometric functions: degree or radian</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal trigonometric_mode As Mode)
        TrigonometricMode = trigonometric_mode
        Functions = New String() {"sin(", "abs(", "cos(", "tan(", "asin(", "acos(", "atan(", "sqrt(", "log(", "ln("}
        Operators = New String() {"+", "-", "*", "/", "%", "^"}
        ValidTokens = New String() {"sin(", "E", "abs(", "cos(", "tan(", "asin(", "acos(", "atan(", "sqrt(", "log(", "ln(", "+", "-", "*", "/", "%", "^", "(", ")"}
    End Sub

    ''' <summary>
    ''' Splits the expression but leaves the seperators in the List
    ''' </summary>
    ''' <param name="expression">The expression which has to split</param>
    ''' <param name="splitter">The seperators, staying</param>
    ''' <returns>The List of the Tokens in infix-notation</returns>
    ''' <remarks></remarks>
    Private Function NewSplit(ByVal expression As String, ByVal splitter As String()) As List(Of Token)
        Dim finallist As New List(Of Token)
        expression = expression.Replace("e", Math.E.ToString).Replace("pi", Math.PI.ToString)
        For Each Split As String In splitter
            expression = expression.Replace(Split, "|" + Split + "|")
        Next
        expression = expression.Replace("|(||", "(|")
        Dim str = expression.Split("|"c)
        For Each Token As String In str
            finallist.Add(New Token(Token))
        Next
        finallist.RemoveAll(Function(item) item.Content = "")
        Return finallist
    End Function

    ''' <summary>
    ''' Puts a number and a minus in one Token if it's appropriate
    ''' </summary>
    ''' <param name="tokens">The Tokenlist requiering correction</param>
    ''' <returns>The corrected Tokenlist</returns>
    ''' <remarks></remarks>
    Private Function CorrectNegotiation(ByVal tokens As List(Of Token)) As List(Of Token)
        If tokens(0).Content = "-" AndAlso tokens(1).Type = TokenType.Number Then
            tokens(1).Content = "-" & tokens(1).Content
            tokens.RemoveAt(0)
        End If
        For index As Integer = 1 To tokens.Count - 2
            If index <= tokens.Count - 1 Then
                If tokens(index).Content = "-" AndAlso tokens(index + 1).Type = TokenType.Number AndAlso (Operators.Contains(tokens(index - 1).Content) OrElse Functions.Contains(tokens(index - 1).Content)) Then
                    tokens(index + 1).Content = "-" & tokens(index + 1).Content
                    tokens.RemoveAt(index)
                End If
                If tokens(index - 1).Content = "(" AndAlso tokens(index).Content = "-" AndAlso tokens(index + 1).Type = TokenType.Number Then
                    tokens(index + 1).Content = "-" & tokens(index + 1).Content
                    tokens.RemoveAt(index)
                End If

            Else
                Exit For
            End If

        Next
        Return tokens
    End Function




    ''' <summary>
    ''' Converts e.g. 2sin() to 2*sin()
    ''' </summary>
    ''' <param name="list">The List which needs to be writed in full</param>
    ''' <returns>The full-written Tokenlist</returns>
    ''' <remarks></remarks>
    Private Function WriteInFull(ByVal list As List(Of Token)) As List(Of Token)
        For u As Integer = 0 To list.Count - 2
            If (list(u).Type = TokenType.Number AndAlso (list(u + 1).Type = TokenType.Function OrElse list(u + 1).Content = "(")) OrElse (list(u).Content = ")" AndAlso list(u + 1).Content = "(") Then
                list.Insert(u + 1, New Token("*"))
            End If
        Next
        Return list
    End Function


    Private Function SolveExponentialProblem(list As List(Of Token)) As List(Of Token)
        Dim result As Double = 0
        While list.Any(Function(item) item.Content = "E")
            For i As Integer = 1 To list.Count - 2
                If list(i).Content = "E" AndAlso list(i - 1).Type = TokenType.Number AndAlso (list(i + 1).Content = "+" OrElse list(i + 1).Content = "-") AndAlso list(i + 2).Type = TokenType.Number Then
                    list(i - 1).Content = list(i - 1).Content + list(i).Content + list(i + 1).Content + list(i + 2).Content
                    list.RemoveRange(i, 3)
                    Exit For
                End If
                If list(i).Content = "E" AndAlso list(i - 1).Type = TokenType.Number AndAlso list(i + 1).Type = TokenType.Number Then
                    list(i - 1).Content = list(i - 1).Content + list(i).Content + list(i + 1).Content
                    list.RemoveRange(i, 2)
                    Exit For
                End If
            Next
        End While
        Return list
    End Function

    ''' <summary>
    ''' Evaluates mathematic expressions
    ''' </summary>
    ''' <param name="Expression">The expression which will be calculated</param>
    ''' <returns>The result of the calculation</returns>
    ''' <remarks></remarks>
    Public Function Evaluate(ByVal Expression As String) As Double
        If String.IsNullOrEmpty(Expression) Then Throw New ArgumentException("Kein gültiger Ausdruck")

        Dim Tokens As List(Of Token) = NewSplit(Expression, ValidTokens)
        Tokens = CorrectNegotiation(Tokens)
        Tokens = SolveExponentialProblem(Tokens)
        If Tokens.Count = 1 Then Return Tokens(0).Value
        Tokens = WriteInFull(Tokens)
        Tokens = ShuntingYard(Tokens)

        While Tokens.Any(Function(item) item.Type = TokenType.Operator OrElse item.Type = TokenType.Function)
            For index As Integer = 0 To Tokens.Count - 1
                If Tokens(index).Type = TokenType.Operator Then
                    Tokens(index).Content = Calculate(Tokens(index).Content, Tokens(index - 2).Value, Tokens(index - 1).Value).ToString
                    Tokens.RemoveRange(index - 2, 2)
                    Exit For
                End If
                If Tokens(index).Type = TokenType.Function Then
                    Tokens(index).Content = Calculate(Tokens(index).Content, Tokens(index - 1).Value, 0).ToString
                    Tokens.RemoveRange(index - 1, 1)
                    Exit For
                End If
            Next
        End While
        Return Tokens(0).Value
    End Function

    ''' <summary>
    ''' Converts a infix-notation Tokenlist to postfix-notation
    ''' </summary>
    ''' <param name="tokens">The infix-notation Tokenlist</param>
    ''' <returns>The postfix-notation Tokenlist</returns>
    ''' <remarks></remarks>
    Private Function ShuntingYard(ByVal tokens As List(Of Token)) As List(Of Token)
        Dim outputList As New List(Of Token)
        Dim operatorStack As New Stack(Of Token)

        For Each tk As Token In tokens

            If tk.Type = TokenType.Number Then
                outputList.Add(tk)
                Continue For
            End If
            If tk.Type = TokenType.Function OrElse tk.Content = "(" Then
                operatorStack.Push(tk)
                Continue For
            End If
            If tk.Content = ")" Then
                Do Until operatorStack.Peek.Type = TokenType.Function OrElse operatorStack.Peek.Content = "(" OrElse operatorStack.Count = 0
                    outputList.Add(operatorStack.Pop)
                Loop
                If operatorStack.Peek.Content <> "(" Then

                    outputList.Add(operatorStack.Pop)
                Else
                    operatorStack.Pop()
                End If
                Continue For
            End If
            If tk.Type = TokenType.Operator AndAlso operatorStack.Count = 0 Then
                operatorStack.Push(tk)
                Continue For
            End If

            If operatorStack.Count > 0 AndAlso tk.Type = TokenType.Operator Then
                While operatorStack.Count > 0 AndAlso tk.EvaluatesBefore(operatorStack.Peek) AndAlso operatorStack.Peek.Content <> "(" AndAlso operatorStack.Peek.Type <> TokenType.Function
                    outputList.Add(operatorStack.Pop)
                End While
                operatorStack.Push(tk)
            End If

        Next
        While operatorStack.Count > 0
            outputList.Add(operatorStack.Pop)
        End While

        Return outputList
    End Function

    ''' <summary>
    ''' Calculates a single arithmetic
    ''' </summary>
    ''' <param name="op">The operator, which will be used</param>
    ''' <param name="left">The Number left of the operator, in case of a function the left will be chosen as the argument</param>
    ''' <param name="right">The Number right of the operator</param>
    ''' <returns>The result of the arithmetic</returns>
    ''' <remarks></remarks>
    Private Function Calculate(ByVal op As String, ByVal left As Double, ByVal right As Double) As Double
        Select Case op
            Case "^"
                Return Math.Pow(left, right)
            Case "*"
                Return left * right
            Case "/"
                Return left / right
            Case "%"
                Return left Mod right
            Case "+"
                Return left + right
            Case "-"
                Return left - right
            Case "sin("
                If TrigonometricMode = Mode.Radian Then Return Math.Sin(left)
                Return Math.Sin(left * Math.PI / 180)
            Case "cos("
                If TrigonometricMode = Mode.Radian Then Return Math.Cos(left)
                Return Math.Cos(left * Math.PI / 180)
            Case "tan("
                If TrigonometricMode = Mode.Radian Then Return Math.Tan(left)
                Return Math.Cos(left * Math.PI / 180)
            Case "asin("
                If TrigonometricMode = Mode.Radian Then Return Math.Asin(left)
                Return Math.Cos(left * Math.PI / 180)
            Case "acos("
                If TrigonometricMode = Mode.Radian Then Return Math.Acos(left)
                Return Math.Cos(left * Math.PI / 180)
            Case "atan("
                If TrigonometricMode = Mode.Radian Then Return Math.Atan(left)
                Return Math.Cos(left * Math.PI / 180)
            Case "abs("
                Return Math.Abs(left)
            Case "log("
                Return Math.Log10(left)
            Case "ln("
                Return Math.Log(left)
            Case "sqrt("
                Return Math.Sqrt(left)
        End Select

    End Function


    ''' <summary>
    ''' Piece of a statement, which could contain a number, an operator or a function
    ''' </summary>
    ''' <remarks></remarks>
    Friend Class Token
        Private Priority As New Dictionary(Of String, Integer)

        Private Associativity As New Dictionary(Of String, Associvity)
        Private _content As String

        Public Property Value As Double
        Public Property Type As TokenType

        Public Property Content As String

            Get
                Return _content
            End Get
            Set(val As String)
                _content = val
                If Double.TryParse(_content, Value) Then
                    Me.Type = TokenType.Number
                ElseIf Operators.Contains(_content) Then
                    Me.Type = TokenType.Operator
                ElseIf Functions.Contains(_content) Then
                    Me.Type = TokenType.Function
                End If
            End Set
        End Property

        ''' <summary>
        ''' Fills the dictionarys required for comparison
        ''' </summary>
        ''' <param name="Tokencontent"></param>
        ''' <remarks></remarks>
        Public Sub New(ByVal Tokencontent As String)
            Content = Tokencontent
            Priority.Add("+", 0)
            Priority.Add("-", 0)
            Priority.Add("*", 1)
            Priority.Add("/", 1)
            Priority.Add("%", 1)
            Priority.Add("^", 2)
            Priority.Add("(", -1)
            Priority.Add(")", -1)
            Priority.Add("sin(", 2)
            Priority.Add("cos(", 2)
            Priority.Add("tan(", 2)
            Priority.Add("asin(", 2)
            Priority.Add("acos(", 2)
            Priority.Add("atan(", 2)
            Priority.Add("sqrt(", 2)
            Priority.Add("log(", 2)
            Priority.Add("ln(", 2)
            Priority.Add("abs(", 2)
            Associativity.Add("+", Associvity.Left)
            Associativity.Add("-", Associvity.Left)
            Associativity.Add("*", Associvity.Left)
            Associativity.Add("/", Associvity.Left)
            Associativity.Add("%", Associvity.Left)
            Associativity.Add("^", Associvity.Right)
            Associativity.Add("(", Associvity.None)
            Associativity.Add(")", Associvity.None)
            Associativity.Add("sin(", Associvity.Right)
            Associativity.Add("cos(", Associvity.Right)
            Associativity.Add("tan(", Associvity.Right)
            Associativity.Add("asin(", Associvity.Right)
            Associativity.Add("acos(", Associvity.Right)
            Associativity.Add("atan(", Associvity.Right)
            Associativity.Add("sqrt(", Associvity.Right)
            Associativity.Add("log(", Associvity.Right)
            Associativity.Add("ln(", Associvity.Right)
            Associativity.Add("abs(", Associvity.Right)
        End Sub

        ''' <summary>
        ''' Compares the priority of Tokens
        ''' </summary>
        ''' <param name="older">The Token compared to the Class</param>
        ''' <returns>Whether the older has a higher Priority, than this Class</returns>
        ''' <remarks></remarks>
        Public Function EvaluatesBefore(ByVal older As Token) As Boolean
            If Priority.Keys.Contains(older.Content) AndAlso Associativity.Keys.Contains(older.Content) AndAlso Priority.Keys.Contains(Content) AndAlso Associativity.Keys.Contains(Content) Then
                Return Priority(older.Content) > Priority(Me.Content) OrElse (Priority(older.Content) = Priority(Me.Content) AndAlso Associativity(Me.Content) = Associvity.Left)
            End If
        End Function
    End Class
End Class



Public Class [Function]
    Private Const H As Double = 0.1000000000000001
    Private Property Variable As String
    Dim pr As New Parser(Mode.Radian)
    Public Property FunctionTerm As String

    ''' <summary>
    ''' Instantiation of a new Function Class
    ''' </summary>
    ''' <param name="term">The term od the function</param>
    ''' <param name="var">The used variable</param>
    ''' <param name="mode">The mode, which the Function Class uses to calculate values</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal term As String, ByVal var As String, ByVal mode As Mode)
        FunctionTerm = term.Trim 'Regex.Replace(term, "([\d,]+[^\-+*])" & var, "$1" & "*" & var).Trim
        pr.TrigonometricMode = mode
        Variable = var
    End Sub


    ''' <summary>
    ''' Calculates the value of the function on the digit x
    ''' </summary>
    ''' <param name="x">The selected digit</param>
    ''' <returns>The value of the Function at x</returns>
    ''' <remarks></remarks>
    Public Function FunctionValue(ByVal x As Double) As Double
        Return (pr.Evaluate(FunctionTerm.Replace(Variable, x.ToString)))
    End Function

    ''' <summary>
    ''' Approximates the first derivation with the limit of h -> 0
    ''' </summary>
    ''' <param name="x">The digit at which the first derivation should be calculated</param>
    ''' <returns>The value of the first derivation, at the digit x</returns>
    ''' <remarks></remarks>
    Public Function FirstDerivationValue(ByVal x As Double) As Double
        Dim h As Double = Math.Pow(10, -7)
        Return ((FunctionValue(x + h) - FunctionValue(x)) / h)
    End Function

    ''' <summary>
    ''' Approximates the second derivation with the limit of h -> 0
    ''' </summary>
    ''' <param name="x">The digit at which the second derivation should be calculated</param>
    ''' <returns>The value of the second derivation, at the digit x</returns>
    ''' <remarks></remarks>
    Public Function SecondDerivationValue(ByVal x As Double) As Double
        Return (FunctionValue(x - H) + FunctionValue(x + H) - 2 * FunctionValue(x)) / H ^ 2
    End Function

    ''' <summary>
    ''' Approximates the third derivation with the limit of h -> 0
    ''' </summary>
    ''' <param name="x">The digit at which the third derivation should be calculated</param>
    ''' <returns>The value of the third derivation, at the digit x</returns>
    ''' <remarks></remarks>
    Public Function ThirdDerivationValue(ByVal x As Double) As Double
        Return (SecondDerivationValue(x + H) - SecondDerivationValue(x)) / H
    End Function


    ''' <summary>
    ''' Calculates extrempoints of the Function
    ''' </summary>
    ''' <param name="guess">The estimated digit of the extremepoint</param>
    ''' <param name="iterations">The number of repetitions of the algorithm (the more the more accurate the result is)</param>
    ''' <param name="accurancy">The number of decimal places, on which the result is rounded to</param>
    ''' <returns>The extremepoint</returns>
    ''' <remarks></remarks>
    Public Function ExtremePoint(ByVal guess As Double, ByVal iterations As Integer, ByVal accurancy As Integer) As ExtremePoint

        For u As Integer = 0 To iterations - 1
            guess = guess - (FirstDerivationValue(guess) / SecondDerivationValue(guess))
        Next

        If Math.Round(FirstDerivationValue(guess), accurancy) = 0 Then


            If FirstDerivationValue(guess - H) > 0 AndAlso FirstDerivationValue(guess + H) < 0 Then
                Return New ExtremePoint With {.Type = PointType.High, .Point = New FunctionPoint(Math.Round(guess, accurancy), FunctionValue(Math.Round(guess, accurancy)))}
            End If

            If FirstDerivationValue(guess - H) < 0 AndAlso FirstDerivationValue(guess + H) > 0 Then
                Return New ExtremePoint With {.Type = PointType.Low, .Point = New FunctionPoint(Math.Round(guess, accurancy), FunctionValue(Math.Round(guess, accurancy)))}
            End If

            If (FirstDerivationValue(guess - H) > 0 AndAlso FirstDerivationValue(guess + H) > 0) OrElse (FirstDerivationValue(guess - H) < 0 AndAlso FirstDerivationValue(guess + H) < 0) Then
                Return New ExtremePoint With {.Type = PointType.Saddle, .Point = New FunctionPoint(Math.Round(guess, accurancy), FunctionValue(Math.Round(guess, accurancy)))}
            End If

        End If

    End Function


    ''' <summary>
    ''' Calculates point of inflection of the Function
    ''' </summary>
    ''' <param name="guess">The estimated digit of the inflectionpoint</param>
    ''' <param name="iterations">The number of repetitions of the algorithm (the more the more accurate the result is)</param>
    ''' <param name="accurancy">The number of decimal places, on which the result is rounded to</param>
    ''' <returns>The inflectionpoint</returns>
    ''' <remarks></remarks>
    Public Function PointOfInflection(ByVal guess As Double, ByVal iterations As Integer, ByVal accurancy As Integer) As InflectionPoint
        For u As Integer = 0 To iterations - 1
            guess = guess - (SecondDerivationValue(guess) / ThirdDerivationValue(guess))
        Next

        Dim before As Double = Math.Round(SecondDerivationValue(guess - 0.0001), accurancy)
        Dim after As Double = Math.Round(SecondDerivationValue(guess + 0.0001), accurancy)

        If before < 0 AndAlso after > 0 Then
            Return New InflectionPoint With {.Point = New FunctionPoint(Math.Round(guess, accurancy), Math.Round(FunctionValue(guess), accurancy)), .CurveChange = CurveChange.RightToLeft}
        End If
        If before > 0 AndAlso after < 0 Then
            Return New InflectionPoint With {.Point = New FunctionPoint(Math.Round(guess, accurancy), Math.Round(FunctionValue(guess), accurancy)), .CurveChange = CurveChange.LeftToRight}
        End If


    End Function


    ''' <summary>
    ''' Calculates the zero digit of a Function
    ''' </summary>
    ''' <param name="guess">The estimated digit</param>
    ''' <param name="iterations">The number of repetitions of the algorithm (the more the more accurate the result is)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Zero(ByVal guess As Double, ByVal iterations As Integer) As Double
        Dim intialguess As Double = guess
        For u As Integer = 0 To iterations - 1
            If FirstDerivationValue(guess) <> 0 Then
                guess = guess - (FunctionValue(guess) / FirstDerivationValue(guess))
            Else
                Return Double.NaN
            End If
        Next
        Return guess
    End Function

    ''' <summary>
    ''' Calculates the tangent of the Function through the digit x
    ''' </summary>
    ''' <param name="x">The digit, through which the Tangent leads</param>
    ''' <returns>The tangent-Function</returns>
    ''' <remarks></remarks>
    Public Function Tangent(ByVal x As Double) As [Function]
        Dim c As Double = FunctionValue(x) - x * FirstDerivationValue(x)
        Return New [Function]((FirstDerivationValue(x).ToString + "*" & Variable & "+" + c.ToString).Replace("+-", "-"), Variable, pr.TrigonometricMode)
    End Function

    ''' <summary>
    ''' Calculates the intersection-digit of this Class with another Function
    ''' </summary>
    ''' <param name="function2">The other Function</param>
    ''' <param name="guess">The estimated digit</param>
    ''' <param name="iterations">The number of repetitions of the algorithm (the more the more accurate the result is)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IntersectsWith(ByVal function2 As [Function], ByVal guess As Double, ByVal iterations As Integer) As Double
        For u = 0 To iterations - 1
            guess = guess - (FunctionValue(guess) - function2.FunctionValue(guess)) / (FirstDerivationValue(guess) - function2.FirstDerivationValue(guess))
        Next
        Return guess
    End Function

    ''' <summary>
    ''' Calculates the integral of a Function
    ''' </summary>
    ''' <param name="a">The lower border</param>
    ''' <param name="b">The upper border</param>
    ''' <param name="deltax">The length of the steps, used to calculate</param>
    ''' <returns>The oriented surface between the Function and the x-axis</returns>
    ''' <remarks></remarks>
    Public Function Integral(ByVal a As Double, ByVal b As Double, ByVal deltax As Double) As Double
        Dim summe As Double = 0
        For u As Double = a To b Step (deltax)
            Dim val = FunctionValue(0.5 * (u + deltax + u))
            If Double.IsNaN(val) = False Then
                summe = summe + val * deltax
            End If
        Next
        Return summe
    End Function

    ''' <summary>
    ''' Calculates the body of rotation of the function in the interval [a;b)
    ''' </summary>
    ''' <param name="a">The lower border</param>
    ''' <param name="b">The upper border</param>
    ''' <param name="deltax">The length of the steps, used to calculate</param>
    ''' <returns>The capacity of the resulting rotationbody of the function</returns>
    ''' <remarks></remarks>
    Public Function RotationBody(ByVal a As Double, ByVal b As Double, ByVal deltax As Double) As Double
        Dim summe As Double = 0
        For u As Double = a To b Step (deltax)
            Dim val = FunctionValue(0.5 * (u + deltax + u))
            If Double.IsNaN(val) = False Then
                summe = summe + Math.Pow(val, 2) * deltax
            End If
        Next
        Return Math.PI * summe
    End Function

    ''' <summary>
    ''' Calculates the average function value in the intervall [a;b)
    ''' </summary>
    ''' <param name="a">The lower border</param>
    ''' <param name="b">The upper border</param>
    ''' <param name="deltax">The length of the steps, used to calculate</param>
    ''' <returns>The average function value</returns>
    ''' <remarks></remarks>
    Public Function ValueAverage(ByVal a As Double, ByVal b As Double, ByVal deltax As Double) As Double
        Return (1 / (a + b)) * Integral(a, b, deltax)
    End Function


    ''' <summary>
    ''' Checks the Function on symmetrical behavior
    ''' </summary>
    ''' <param name="iterations">The number of repetitions of the algorithm (the more the more accurate the result is)</param>
    ''' <returns>The symmetry of the Function</returns>
    ''' <remarks></remarks>
    Public Function Symmetry(ByVal iterations As Integer) As Symmetry
        'f(x)=f(-x) ; f(x)-f(-x)=0 ; => axis
        'f(x)=-f(-x) ; f(x)+f(-x)=0 ; => origin

        Try
            Dim axisfunction As New [Function](FunctionTerm & "-" & Regex.Replace(FunctionTerm, Variable, "-" & Variable).Replace("--", ""), Variable, Mode.Radian)
            If axisfunction.Zero(2.0342, 12) = 2.0342 Then
                Return MathExpressionLib.Symmetry.Axis
            End If
        Catch ex As Exception

        End Try


        Try
            Dim originfunction As New [Function](FunctionTerm & "+" & Regex.Replace(FunctionTerm, Variable, "-" & Variable).Replace("--", ""), Variable, Mode.Radian)
            If originfunction.Zero(2.0342, 12) = 2.0342 Then
                Return MathExpressionLib.Symmetry.Origin
            End If
        Catch ex As Exception

        End Try

        Return MathExpressionLib.Symmetry.None
    End Function

End Class




Public Enum ApproximationBehavior
    TowardsMinusInfinty
    TowardsPlusInfinty
End Enum

Public Class VerticalAsymptote
    Public Property Digit As Double

    Public Property ApproximationBehaviorFromLeft As ApproximationBehavior

    Public Property ApproximationBehaviorFromRight As ApproximationBehavior

    Public Sub New(ByVal digit As Double, ByVal behaviorfromleft As ApproximationBehavior, ByVal behaviorfromright As ApproximationBehavior)
        _Digit = digit
        ApproximationBehaviorFromLeft = behaviorfromleft
        ApproximationBehaviorFromRight = behaviorfromright
    End Sub
    Public Overrides Function ToString() As String
        Return "VerticalAsymptote at x=" & Digit & Environment.NewLine & _
               "x->" & Digit & " ; x<" & Digit & " ; f(x)->" & ApproximationBehaviorFromLeft.ToString & Environment.NewLine & _
               "x->" & Digit & " ; x>" & Digit & " ; f(x)->" & ApproximationBehaviorFromRight.ToString
    End Function
End Class

Public Class RationalFunction
    Inherits [Function]

    Shared Property NumeratorFunction As [Function]
    Shared Property DenominatorFunction As [Function]

    Shared Property Mode As Mode

    Shared Property Variable As String
    Public Sub New()
        MyBase.New("1", "x", MathExpressionLib.Mode.Radian)
    End Sub
    Public Sub New(ByVal numeratorterm As String, ByVal denominatorterm As String, ByVal var As String, ByVal mode As Mode)
        MyBase.New("(" & numeratorterm & ")/(" & denominatorterm & ")", var, mode)
        NumeratorFunction = New [Function](numeratorterm, var, mode)
        DenominatorFunction = New [Function](denominatorterm, var, mode)
        Variable = var
        _Mode = mode
    End Sub


    ''' <summary>
    ''' Splits the BrokenRationalTerm at the fitting spot
    ''' </summary>
    ''' <param name="term">the term which shpuld be splitted</param>
    ''' <returns>A string array with the term parts</returns>
    ''' <remarks></remarks>
    Shared Function SplitBrokenRational(ByVal term As String) As String()
        Dim Level As Integer = 0
        Dim Lines As Integer = 0
        For index As Integer = 0 To term.Length - 1
            Select Case term(index)
                Case Is = "("c
                    Level += 1
                Case Is = ")"c
                    Level -= 1
                Case Is = "/"c
                    If Level = 0 Then Return {term.Substring(0, index), term.Substring(index + 1)}
                    Lines += 1
            End Select
        Next
        Return Nothing
    End Function

    ''' <summary>
    ''' Function to parse String like (1)/(x) into a BrokenRationalFunction
    ''' </summary>
    ''' <param name="object">The String which should be converted</param>
    ''' <param name="brokenrational">The BrokenRationalFunction which will receive the parsed terms</param>
    ''' <param name="var">The used variable</param>
    ''' <param name="mode">The Mode the Function is supposed to use</param>
    ''' <returns>If the conversion was succesfully</returns>
    ''' <remarks></remarks>
    Shared Function TryParse(ByVal [object] As String, ByVal var As String, ByVal mode As Mode, ByRef brokenrational As RationalFunction) As Boolean
        Dim seperateterms As String() = SplitBrokenRational([object])

        If seperateterms.Count > 2 Then
            Return False
        Else
            Variable = var
            _Mode = mode
            NumeratorFunction = New [Function](seperateterms(0), Variable, mode)
            DenominatorFunction = New [Function](seperateterms(1), Variable, mode)
            brokenrational = New RationalFunction(seperateterms(0), seperateterms(1), Variable, mode)
            Return True
        End If
    End Function


    ''' <summary>
    ''' Converts a String in a BrokenRationalFunction
    ''' </summary>
    ''' <param name="object">The String which should be converted</param>
    ''' <param name="var">The used variable</param>
    ''' <param name="mode">The Mode the Function is supposed to use</param>
    ''' <returns>The BrokenRationalFunction</returns>
    ''' <remarks></remarks>
    Shared Function Parse(ByVal [object] As String, ByVal var As String, ByVal mode As Mode) As RationalFunction
        Dim seperateterms As String() = SplitBrokenRational([object])

        If seperateterms.Count = 2 Then
            Variable = var
            _Mode = mode
            NumeratorFunction = New [Function](seperateterms(0), Variable, mode)
            DenominatorFunction = New [Function](seperateterms(1), Variable, mode)
            Return New RationalFunction(seperateterms(0), seperateterms(1), Variable, mode)
        End If
        Return Nothing
    End Function

    ''' <summary>
    ''' Calculates the Vertical Asymptotes of a Function 
    ''' </summary>
    ''' <param name="guess">The estimated digit of the Asymptote</param>
    ''' <param name="iterations">The number of repetitions of the algorithm (the more the more accurate the result is)</param>
    ''' <param name="accurancy">The number of decimal places, on which the result is rounded to</param>
    ''' <returns>The VerticalAsymptote</returns>
    ''' <remarks></remarks>
    Public Function VerticalAsymptote(ByVal guess As Double, ByVal iterations As Integer, ByVal accurancy As Integer) As VerticalAsymptote
        Dim approxbehavleft As ApproximationBehavior
        Dim approxbehavright As ApproximationBehavior
        Dim denominatorzero As Double = DenominatorFunction.Zero(guess, iterations)
        If Math.Round(NumeratorFunction.FunctionValue(guess), accurancy) <> 0 Then
            denominatorzero = Math.Round(denominatorzero, accurancy)
            If MyBase.FunctionValue(denominatorzero - 0.01) < 0 Then
                approxbehavleft = ApproximationBehavior.TowardsMinusInfinty
            Else
                approxbehavleft = ApproximationBehavior.TowardsPlusInfinty
            End If

            If MyBase.FunctionValue(denominatorzero + 0.01) < 0 Then
                approxbehavright = ApproximationBehavior.TowardsMinusInfinty
            Else
                approxbehavright = ApproximationBehavior.TowardsPlusInfinty
            End If

            Return New VerticalAsymptote(denominatorzero, approxbehavleft, approxbehavright)
        End If
    End Function

End Class


